# G46-MP1: The Lord of Logs

## Introduction

In MP1, we use Rust to implement a distributed log search application using sockets. The application consists of a server and a client that communicate over TCP. The server runs on multiple virtual machines (VMs), each hosting a log file, and the client can search for patterns across these log files concurrently.

## Project Structure

The main programs are:

- `server.rs` — The server program that runs on each VM.
- `client.rs` — The client program that connects to the servers to perform the log search.

Both files are located in `g46-mp1/MP1/src/bin`.

## Prerequisites

- **Rust** installed on all VMs (`rustc` and `cargo`)
- Access to the VMs
- Log files (`vmi.log`) available on each VM

## Setting Up the Log Files

For each VM, download and set up the log files by running:

```bash
wget https://courses.engr.illinois.edu/cs425/fa2024/MPDemos/MP1DemoDataFA24.zip
unzip MP1DemoDataFA24.zip
cd "MP1 Demo Data FA24"
mv vmX.log ../
cd ..
rm -r "MP1 Demo Data FA24"
rm MP1DemoDataFA24.zip
```

Replace `vmX.log` with the appropriate log file for the VM (e.g., `vm1.log` for VM1, `vm2.log` for VM2, etc.).

## Running the Server

On each VM, navigate to the `MP1` directory:

```bash
cd g46-mp1/MP1
```

Run the server:

```bash
cargo run --bin server
```

This command starts the server on the default port `7878`. The server listens for incoming connections from clients.

**Note:** Ensure that this command is running on all VMs you wish to include in the search.

## Running the Client

To perform a log search, SSH into any VM (preferably one with the client code) and navigate to the `MP1` directory:

```bash
cd g46-mp1/MP1
```

Run the client with the desired search pattern:

```bash
cargo run --bin client "pattern"
```

Replace `"pattern"` with the specific text or regular expression you want to search for. The pattern should be enclosed in quotes.

For example, to search for the term `GET`, run:

```bash
cargo run --bin client "GET"
```

The client will connect to all VMs listed in the `HOSTS` array within `client.rs`, send the search pattern, and collect the results.

## Example Usage

### Searching for a Common Term

To search for a term that appears frequently in the logs, such as `GET`, run:

```bash
cargo run --bin client "GET"
```

Sample output:

```
File name: vm3.log
File name: vm2.log
From Machine: fa24-cs425-4603.cs.illinois.edu, 161169 lines matched, took 8.1896 seconds
From Machine: fa24-cs425-4602.cs.illinois.edu, 160563 lines matched, took 8.2300 seconds
File name: vm4.log
File name: vm5.log
From Machine: fa24-cs425-4604.cs.illinois.edu, 162590 lines matched, took 9.6896 seconds
From Machine: fa24-cs425-4605.cs.illinois.edu, 162714 lines matched, took 9.7466 seconds
File name: vm1.log
From Machine: fa24-cs425-4601.cs.illinois.edu, 170372 lines matched, took 10.8892 seconds

🦀==========  Covfefe Software with Rust  ==========🦀
Total 817408 lines matched across all hosts, total time: 10.8892 seconds
```
```
grep -c "GET"
```

The following matched lines from our program get written to a .temp file where we test the accuracy of our program 
by running the following command using the built in `grep -c "GET" file` within a bash script in our test folder

```
From Machine: fa24-cs425-4604.cs.illinois.edu, 162590 lines matched
From Machine: fa24-cs425-4605.cs.illinois.edu, 162714 lines matched
From Machine: fa24-cs425-4601.cs.illinois.edu, 170372 lines matched
From Machine: fa24-cs425-4603.cs.illinois.edu, 161169 lines matched
From Machine: fa24-cs425-4602.cs.illinois.edu, 160563 lines matched
```

We performed the following test with the actual `vmi.log` for all i from 1 to 5 storing the log files temporarly in one of the vms
the script can be located in the test folder but we did remove the log files afer testing.

```
From Machine: vm2.log, 160563 lines matched
From Machine: vm4.log, 162590 lines matched
From Machine: vm1.log, 170372 lines matched
From Machine: vm3.log, 161169 lines matched
From Machine: vm5.log, 162714 lines matched
```


### Searching with Regular Expressions

You can also use regular expressions. For example, to search for lines containing date and time inside square brackets, run:

```bash
cargo run --bin client "\[.*\]"
```

Sample output:

```
File name: vm3.log
File name: vm2.log
From Machine: fa24-cs425-4603.cs.illinois.edu, 268804 lines matched, took 17.1490 seconds
From Machine: fa24-cs425-4602.cs.illinois.edu, 267938 lines matched, took 17.2148 seconds
File name: vm5.log
File name: vm4.log
From Machine: fa24-cs425-4605.cs.illinois.edu, 271205 lines matched, took 17.5442 seconds
From Machine: fa24-cs425-4604.cs.illinois.edu, 270917 lines matched, took 17.5992 seconds
File name: vm1.log
From Machine: fa24-cs425-4601.cs.illinois.edu, 283553 lines matched, took 17.8495 seconds

🦀==========  Covfefe Software with Rust  ==========🦀
Total 1362417 lines matched across all hosts, total time: 17.8495 seconds
```

## Cleaning Up

The client generates temporary files (`*.temp`) during the search process. To remove these files, run:

```bash
rm *.temp
```

Alternatively, the client program automatically cleans up old temporary files at the start of each run.

## Notes

- Ensure that the server is running on all VMs listed in the `HOSTS` array in `client.rs`. If some VMs are down or the server is not running, the client will display an error for those VMs.
- The default port used is `7878`. If you need to change the port, update the `PORT` constant in both `server.rs` and `client.rs`.
- Make sure that firewall settings on the VMs allow for TCP connections on the specified port.
- The client connects to the hosts concurrently using threads to speed up the search process.

## Testing

### Test 1: Search for Common Term

**Objective:** Verify that the client can search for a common term across multiple VMs.

**Procedure:**

1. Ensure that the server is running on at least 5 VMs.
2. On one of the VMs (e.g., VM3), run the client:

   ```bash
   cargo run --bin client "GET"
   ```

3. Observe the output, which should display the number of matches found on each VM and the time taken.

Example Output: 

```
From Machine: fa24-cs425-4601.cs.illinois.edu, 1234 lines matched, took 0.5678 seconds
From Machine: fa24-cs425-4602.cs.illinois.edu, 1156 lines matched, took 0.4890 seconds
...

🦀==========  Covfefe Software with Rust  ==========🦀
Total 6789 lines matched across all hosts, total time: 1.2345 seconds
```

**Expected Result:**

- The client successfully connects to all running servers.
- The output displays the number of lines containing the term `GET` on each VM.
- The total number of matches and total time are displayed at the end.

### Test 2: Search with Regular Expression

**Objective:** Verify that the client can handle regular expressions.

**Procedure:**

1. Run the client with a regex pattern:

   ```bash
   cargo run --bin client "Error|Warning|Failed"
   ```

2. Observe the output.

**Expected Result:**

- The client finds lines containing any of the terms `Error`, `Warning`, or `Failed`.
- The output displays the number of matches on each VM.

### Test 3: No Matches Found

**Objective:** Verify the client's behavior when no matches are found.

**Procedure:**

1. Run the client with a pattern that does not exist in the logs:

   ```bash
   cargo run --bin client "NonExistentPattern123"
   ```

2. Observe the output.

**Expected Result:**

- The client reports zero matches on each VM.
- The total number of matches is zero.

### Test 4: Server Not Running

**Objective:** Verify the client's behavior when some servers are not running.

**Procedure:**

1. Stop the server on one or more VMs.
2. Run the client:

   ```bash
   cargo run --bin client "GET"
   ```

3. Observe the output.

Example Output with details of `FAILED` connections

```
[ERROR] Failed to connect to fa24-cs425-4608.cs.illinois.edu: Connection refused (os error 111)
[ERROR] Failed to connect to fa24-cs425-4606.cs.illinois.edu: Connection refused (os error 111)
[ERROR] Failed to connect to fa24-cs425-4607.cs.illinois.edu: Connection refused (os error 111)
[ERROR] Failed to connect to fa24-cs425-4610.cs.illinois.edu: Connection refused (os error 111)
[ERROR] Failed to connect to fa24-cs425-4609.cs.illinois.edu: Connection refused (os error 111)
File name: vm3.log
File name: vm2.log
From Machine: fa24-cs425-4603.cs.illinois.edu, 268804 lines matched, took 17.1490 seconds
From Machine: fa24-cs425-4602.cs.illinois.edu, 267938 lines matched, took 17.2148 seconds
File name: vm5.log
File name: vm4.log
From Machine: fa24-cs425-4605.cs.illinois.edu, 271205 lines matched, took 17.5442 seconds
From Machine: fa24-cs425-4604.cs.illinois.edu, 270917 lines matched, took 17.5992 seconds
File name: vm1.log
From Machine: fa24-cs425-4601.cs.illinois.edu, 283553 lines matched, took 17.8495 seconds

🦀==========  Covfefe Software with Rust  ==========🦀
Total 1362417 lines matched across all hosts, total time: 17.8495 seconds
```

**Expected Result:**

- The client reports errors connecting to the VMs where the server is not running.
- The client continues to receive results from the VMs where the server is running.

## Contributors

- Jorge Velez (jorgev2)
- Vashishth Goswami (vgosw2)



